from fastapi import APIRouter
from fastapi.exceptions import HTTPException
from os import environ
from pydantic import BaseModel

from mcrcon import MCRconException
from mcapi.rcon import RconActions

# Minecraft RCON connection vars
MC_HOST = environ["MC_HOST"]
RCON_PORT = int(environ["RCON_PORT"])


# Routers settings``
rcon = APIRouter(prefix="/v2", tags=["APIv2", "Server", "RCON"])


@rcon.post("/restart")
async def restart(password: str):
    async with RconActions(MC_HOST, RCON_PORT, password) as rcon:
        try:
            await rcon.restart()
            return {"status": True}
        except MCRconException:
            raise HTTPException(status_code=403, detail="Unauthorized")


class Title(BaseModel):
    text: str
    color: str
    subtext: str
    subcolor: str


@rcon.post("/title")
async def title(password: str, content: Title):
    text, color, subtext, subcolor = (
        content.text,
        content.color,
        content.subtext,
        content.subcolor,
    )
    async with RconActions(MC_HOST, RCON_PORT, password) as rcon:
        try:
            await rcon.title(text, color, subtext, subcolor)
        except MCRconException:
            raise HTTPException(status_code=403, detail="Unauthorized")


@rcon.post("/say")
async def say(password: str, content: str):
    async with RconActions(MC_HOST, RCON_PORT, password) as rcon:
        try:
            await rcon.say(content)
            return True
        except MCRconException:
            raise HTTPException(status_code=403, detail="Unauthorized")
