from fastapi import FastAPI
from mcapi.APIv1 import default as v1_default, players as v1_players
from mcapi.APIv2 import rcon
from modcards.api_module import default as modcard_api
from fastapi.middleware.cors import CORSMiddleware
from os import environ
from redis import asyncio as aioredis
from fastapi_cache import FastAPICache
from fastapi_cache.backends.redis import RedisBackend

REDIS_HOST = environ["REDIS"]


origins = ["*"]

api = FastAPI()


@api.on_event("startup")
async def startup():
    redis = aioredis.from_url(f"redis://{REDIS_HOST}")
    FastAPICache.init(RedisBackend(redis), prefix="fastapi-cache")


api.add_middleware(
    CORSMiddleware,
    allow_origins=origins,
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# APIv1
api.include_router(v1_default)
api.include_router(v1_players)

# APIv2
api.include_router(rcon)

# APIv3
api.include_router(modcard_api)
