from collections.abc import Iterable
from mcrcon import MCRcon
import json
from asyncio import sleep, gather


class RconActions(object):
    def __init__(self, host: str, port: int, passw: str):
        self.host = host
        self.port = port
        self.passw = passw

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb) -> None:
        pass

    async def command(self, command: str | Iterable, slp: int | None = None):
        with MCRcon(host=self.host, port=self.port, password=self.passw) as mc:
            if isinstance(command, str):
                mc.command(command)
                return
            for cmd in command:
                if slp is not None:
                    await sleep(slp)
                if isinstance(cmd, str):
                    mc.command(cmd)
                else:
                    for i in cmd:
                        mc.command(i)

    async def title(
        self,
        text: str,
        color: str,
        subtext: str,
        subcolor: str,
        times: tuple = ("20", "100", "20"),
        exec: bool = True,
    ):
        arg = {"color": color, "bold": True, "text": text}
        subarg = {"color": subcolor, "italic": True, "text": subtext}
        commands = []
        commands.append(f"title @a times {' '.join(times)}")
        commands.append(f"title @a subtitle {json.dumps(subarg)}")
        commands.append(f"title @a title {json.dumps(arg)}")
        if exec:
            await self.command(commands)
        return commands

    async def __restart_dlya_pendosov(self):
        cmds = [
            await self.say(f"Server will restart in {i}...", False)
            for i in (5, 4, 3, 2, 1)
        ]
        await self.command(cmds, 1)

    async def restart(self):
        cmds = []
        cmds.append(
            await self.title(
                "Рестарт сервера через 5",
                "red",
                "Вы будете переподключены автоматически",
                "yellow",
                ("20", "40", "0"),
                False,
            )
        )
        for i in (4, 3, 2):
            cmds.append(
                await self.title(
                    f"Рестарт сервера через {i}",
                    "red",
                    "Вы будете переподключены автоматически",
                    "yellow",
                    ("0", "40", "0"),
                    False,
                )
            )
        cmds.append(
            await self.title(
                "Рестарт сервера через 1",
                "red",
                "Вы будете переподключены автоматически",
                "yellow",
                ("0", "40", "20"),
                False,
            )
        )
        cmds.append("stop")
        tasks = [self.command(cmds, 1), self.__restart_dlya_pendosov()]
        await gather(*tasks)

    async def say(self, text: str, exec: bool = True):
        cmd = f"say {text}"
        if exec:
            await self.command(cmd)
        return cmd
