from fastapi import APIRouter
from fastapi.exceptions import HTTPException
from modcards.functions import get_card
from fastapi.responses import StreamingResponse
from io import StringIO
from redis import asyncio as aioredis
from os import environ

redis_ip = environ["REDIS"]

default = APIRouter(prefix="/mod", tags=["APIv3", "Modcards"])


@default.get("/getcard")
async def getcard(mod: str):
    # redis = await aioredis.create_redis_pool(f"redis://{redis_ip}")
    redis = await aioredis.from_url(f"redis://{redis_ip}", encoding="utf-8", decode_responses=True)
    try:
        svg = await redis.get(mod)
        if svg is None:
            svg = await get_card(f"https://modrinth.com/mod/{mod}")
            await redis.set(mod, svg, ex=43200)
        await redis.close()
        output = StringIO(svg)
        return StreamingResponse(output, media_type="image/svg+xml", headers={"Content-Disposition": "attachment; filename=card.svg"})
    except Exception as e:
        raise HTTPException(status_code=502, detail=f"{e}")
