from aiohttp import ClientSession
from bs4 import BeautifulSoup, Tag


class Modrinth(object):

    page: BeautifulSoup | None = None

    def __init__(self, mod_url: str) -> None:
        self.mod = mod_url

    async def __getsmth(self, name: str, cls: str) -> Tag:
        if isinstance(self.page, BeautifulSoup):
            item = self.page.find(name, class_=cls)
            if isinstance(item, Tag):
                return item
        raise ValueError

    @property
    async def icon(self) -> str:
        item = await self.__getsmth("img", "avatar size-md no-shadow project__icon")
        return str(item["src"])

    @property
    async def description(self) -> str:
        item = await self.__getsmth("p", "description")
        return item.text

    @property
    async def name(self) -> str:
        item = await self.__getsmth("h1", "title")
        return item.text

    async def update(self):
        async with ClientSession() as session:
            response = await session.get(self.mod)
            if response.status != 200:
                raise ConnectionError
            html = await response.text()
            self.page = BeautifulSoup(html, "html.parser")

    @property
    async def all(self) -> dict:
        return {
                "name": await self.name,
                "description": await self.description,
                "icon": await self.icon
                }


async def get_mod(mod: str) -> dict:
    modrinth = Modrinth(mod)
    await modrinth.update()
    return await modrinth.all
