import svgwrite
from textwrap import wrap


class ModCard(object):

    def __init__(self, title: str, desc: str, img: str):
        self.title = title
        self.description = desc
        self.img = img
        self.dwg = self.__generate()

    lpad = 150
    rpad = 50
    tbpad = 50
    tlpad = 250
    trpad = 100
    ttpad = 80
    tbotpad = 100
    radius = 30
    title_font_size = 130
    title_font_weight = 700
    desc_font_size = 60
    desc_font_weight = 600
    pargap = 20
    tgap = 40

    @staticmethod
    def __calc_txt_params(text_lines, font_size, vertical_padding, max_width):
        wrapped_lines = []
        for line in text_lines:
            wrapped_lines.extend(wrap(line, width=int(max_width / (font_size * 0.6))))
        max_text_width = max(len(line) for line in wrapped_lines) * font_size * 0.6
        total_text_height = (len(wrapped_lines) * font_size) + (len(wrapped_lines) - 1) * vertical_padding
        return max_text_width, total_text_height

    def __generate(self):
        max_text_width_foobar, total_text_height_foobar = self.__calc_txt_params(
            self.title.split('\n'),
            self.title_font_size,
            self.pargap,
            10000
        )

        max_text_width_description, total_text_height_description = self.__calc_txt_params(
            self.description.split('\n'),
            self.desc_font_size,
            self.pargap,
            10000
        )

        total_text_height = total_text_height_foobar + self.tgap + total_text_height_description

        inner_width = self.tlpad + self.trpad + max(max_text_width_foobar, max_text_width_description)
        inner_height = self.ttpad + total_text_height + self.tbotpad

        width = self.lpad + inner_width + self.rpad

        height = self.tbpad + inner_height + self.tbpad

        dwg = svgwrite.Drawing('gradient_rectangle.svg', profile='full', size=(width, height))

        style_content = """
        @import url('https://fonts.googleapis.com/css2?family=Montserrat:wght@600;700&display=swap');

        text {
            font-family: 'Montserrat', sans-serif;
        }
        """
        dwg.defs.add(dwg.style(type="text/css", content=style_content))

        linear_gradient = dwg.linearGradient(start=(0, 0), end=(1, 0))
        linear_gradient.add_stop_color(0, '#d56d4d')
        linear_gradient.add_stop_color(1, '#c44c4f')
        dwg.defs.add(linear_gradient)

        dwg.add(dwg.rect(insert=(0, 0), size=(width, height), fill=linear_gradient.get_paint_server(), rx=self.radius,
                         ry=self.radius))

        inner_x = self.lpad
        inner_y = self.tbpad

        dwg.add(dwg.rect(insert=(inner_x, inner_y), size=(inner_width, inner_height), fill='white', rx=self.radius,
                         ry=self.radius))

        square_x = self.lpad - 100
        square_y = (height - 300) / 2

        dwg.add(dwg.rect(
            insert=(square_x, square_y),
            size=(300, 300),
            fill='white',
            rx=self.radius,
            ry=self.radius,
        ))

        dwg.add(dwg.image(
            href=self.img,
            insert=(square_x, square_y),
            size=(300, 300),
            preserveAspectRatio='xMidYMid slice'
        ))

        dwg.add(dwg.rect(
            insert=(square_x, square_y),
            size=(300, 300),
            fill='none',
            rx=self.radius,
            ry=self.radius,
            stroke='white',
            stroke_width=20
        ))

        text_x = inner_x + self.tlpad
        text_y = inner_y + self.ttpad

        current_y = text_y
        for line in self.title.split('\n'):
            text_foobar = dwg.text(
                line,
                insert=(text_x, current_y + self.title_font_size),
                fill=linear_gradient.get_paint_server(),
                font_size=self.title_font_size,
                font_weight=self.title_font_weight
            )
            dwg.add(text_foobar)
            current_y += self.title_font_size + self.pargap

        current_y += self.tgap - self.pargap

        for line in self.description.split('\n'):
            text_description = dwg.text(
                line,
                insert=(text_x, current_y + self.desc_font_size),
                fill=linear_gradient.get_paint_server(),
                font_size=self.desc_font_size,
                font_weight=self.desc_font_weight
            )
            dwg.add(text_description)
            current_y += self.desc_font_size + self.pargap

        return dwg

    def save_to_disc(self, filename: str) -> None:
        self.dwg.saveas(filename)

    def __str__(self) -> str:
        return self.dwg.tostring()


def generate_mod_card(modname: str, mod_description: str, icon_link: str) -> ModCard:
    return ModCard(modname, mod_description, icon_link)
